import Gauge from "gaugeJS";

export async function _default(fields, params) {
  // Default values for Gauge.js settings
  const angle = fields.gauge_angle || 0.15;
  const lineWidth = fields.gauge_line_width || 0.44;
  const pointerLength = fields.gauge_pointer_length || 0.9;
  const pointerStrokeWidth = fields.gauge_pointer_stroke_width || 0.035;
  const colorStart = fields.gauge_color_start || "#6FADCF";
  const colorStop = fields.gauge_color_stop || "#8FC0DA";
  const strokeColor = fields.gauge_stroke_color || "#E0E0E0";
  const maxValue = fields.gauge_max_value || 3000;
  const minValue = fields.gauge_min_value || 0;
  const gaugeValue = fields.gauge_value || 1250;

  // Unique ID for the gauge container
  const id = params.plugin_name + "_" + func.utils.makeid(20);

  // Create a container for the gauge
  $(`<canvas id="${id}">`).appendTo(params.$containerP);

  // Gauge.js configuration options
  const opts = {
    angle: angle, // Arc span
    lineWidth: lineWidth, // Thickness of the arc
    pointer: {
      length: pointerLength, // Pointer length relative to gauge radius
      strokeWidth: pointerStrokeWidth, // Pointer thickness
    },
    colorStart: colorStart, // Gradient start color
    colorStop: colorStop, // Gradient stop color
    strokeColor: strokeColor, // Background arc color
  };

  // Initialize the gauge
  const target = document.getElementById(id);
  const gauge = new Gauge.Gauge(target).setOptions(opts);
  gauge.maxValue = maxValue; // Set maximum value
  gauge.setMinValue(minValue); // Set minimum value
  gauge.set(gaugeValue); // Set the initial value

  // Save gauge instance in fields for potential dynamic updates
  fields.gauge_instance = gauge;
}
