import { resolve, basename } from "path";

import { cpSync } from "fs";

export default {
  root: "./tests",
  build: {
    lib: {
      entry: "src/runtime.mjs",
      formats: ["es"],
      fileName: (format, entryName) => `${entryName}.mjs`,
    },
    rollupOptions: {
      input: "src/runtime.mjs",
      output: [
        {
          dir: "dist",
          format: "es",
          exports: "named",
          entryFileNames: "[name].mjs",
          name: "MyLibrary",
          inlineDynamicImports: true,

          assetFileNames: (assetInfo) => {
            if (assetInfo.name && assetInfo.name.endsWith(".css")) {
              return "runtime[extname]";
            }
            return "[name]-[hash][extname]";
          },
        },
      ],
    },
    emptyOutDir: true,
  },
  plugins: [
    {
      name: "copy-index-file",
      writeBundle() {
        cpSync(resolve(__dirname, "src"), resolve("dist"), {
          recursive: true,
          filter: (srcPath) => !srcPath.endsWith("runtime.mjs"),
        });
      },
    },
  ],
  define: {
    "process.env": process.env,
  },
};
